Screen {
    background: $surface-darken-1;
}

#browser-view {
    display: none; 
    scrollbar-gutter: stable;
    overflow: auto;    
    width: auto;    
    height: 100%;            
    dock: left;       
}

Jpterm.-show-browser #browser-view {
    display: block;        
    max-width: 50%;
}

#main-view {
    min-width: 100%;        
    overflow: auto auto;
}
